<?php

namespace Drupal\bibcite_import_orcid;

use Drupal\bibcite\HumanNameParser;
use Drupal\user\Entity\User;

/**
 * Import ORCID data.
 */
class Import {
  /**
   * Process.
   *
   * @var \Drupal\bibcite_import_orcid\Process
   */
  protected $process;

  /**
   * Construct.
   */
  public function __construct() {
    $this->process = new Process();
  }

  /**
   * Import Publications.
   *
   * @return array
   *   The context with created or updated publications.
   */
  public function importOrcid($work, $uid, &$context) {
    $reference = \Drupal::entityTypeManager()->getStorage('bibcite_reference');
    $user = \Drupal::entityTypeManager()->getStorage('user')->load($uid);
    // Check if the user is syncing authors.
    if (\Drupal::config('bibcite_import_orcid.settings')->get('orcid_sync_authors')) {
      $work_details = json_decode($work['pub'], TRUE);
      $selected_author = $work['author'];
      $pub_authors = [];
      if (isset($selected_author)) {
        foreach ($work_details["contribs"] as $key => $value) {
          $existing_users = $work_details["contribs_atts"] ? array_keys($work_details["contribs_atts"]) : [];
          $id = in_array($key, $existing_users) ? $key : $this->getIdFromAuthor($value, $uid);
          $pub_authors[] = intval($id);
          if ($key == $selected_author) {
            $authors = $user->field_author->getValue();
            $current_author_ids = isset($authors) ? array_column($authors, 'target_id') : [];
            if (!in_array($id, $current_author_ids)) {
              $user->field_author[] = ['target_id' => $id];
              $user->save();
            }
          }
        }
      }
    }
    else {
      $work_details = json_decode($work, TRUE);
      if ($work_details["contribs"]) {
        $pub_authors = [];
        foreach ($work_details["contribs"] as $key => $value) {
          $pub_authors[] = $this->getIdFromAuthor($value, $uid);
        }
      }
    }
    $title = $work_details['title'];
    $pub_id = $this->process->getExistingPublication($title, $work_details['type'], $work_details["bibcite_doi"]);

    // If new publication, create it.
    if (empty($pub_id)) {
      $bibcite_reference = $reference->create($work_details);
      $context['results']['create'][] = $title;
      // Set unublished if setting is enabled.
      $default_status = \Drupal::config('bibcite_import_orcid.settings')->get('orcid_unpub_default');
      $default_status ? $bibcite_reference->status = 0 : NULL;
    }
    else {
      // Load existing and update info if it exists.
      $bibcite_reference = $reference->load($pub_id);
      isset($work_details["bibtex"]["bibcite_pages"]) ? $bibcite_reference->bibcite_pages->setValue($work_details["bibtex"]["bibcite_pages"]) : NULL;
      isset($work_details["secundary_title"]) ? $bibcite_reference->bibcite_secondary_title->setValue($work_details["secundary_title"]) : NULL;
      isset($work_details["bibtex"]["bibcite_booktitle"]) ? $bibcite_reference->bibcite_secondary_title->setValue($work_details["bibtex"]["bibcite_booktitle"]) : NULL;
      isset($work_details["bibtex"]["bibcite_volume"]) ? $bibcite_reference->bibcite_volume->setValue($work_details["bibtex"]["bibcite_volume"]) : NULL;
      isset($work_details["bibtex"]["bibcite_section"]) ? $bibcite_reference->bibcite_section->setValue($work_details["bibtex"]["bibcite_section"]) : NULL;
      isset($work_details["bibtex"]["bibcite_publisher"]) ? $bibcite_reference->bibcite_publisher->setValue($work_details["bibtex"]["bibcite_publisher"]) : NULL;
      isset($work_details["bibcite_url"]) ? $bibcite_reference->bibcite_url->setValue($work_details["bibcite_url"]) : NULL;
      isset($work_details["bibcite_other_number"]) ? $bibcite_reference->bibcite_other_number->setValue($work_details["bibcite_other_number"]) : NULL;
      isset($work_details["bibcite_issn"]) ? $bibcite_reference->bibcite_issn->setValue($work_details["bibcite_issn"]) : NULL;
      isset($work_details["bibcite_isbn"]) ? $bibcite_reference->bibcite_isbn->setValue($work_details["bibcite_isbn"]) : NULL;
      isset($work_details["bibcite_doi"]) ? $bibcite_reference->bibcite_doi->setValue($work_details["bibcite_doi"]) : NULL;
      $context['results']['update'][] = $title;
    }
    if (!\Drupal::config('bibcite_import_orcid.settings')->get('orcid_sync_authors')) {
      $this->addPublicationToUser($user, $pub_id);
    }
    $pub_authors ? $bibcite_reference->author = $pub_authors : NULL;
    $bibcite_reference->save();
    return $context;
  }

  /**
   * Given a author name and $uid, look for an author.
   *
   * @return int
   *   The id if found, else create a new author and return the id
   */
  public function getIdFromAuthor(string $name, int $uid) {
    $humanNameParser = new HumanNameParser();
    $parsed = $humanNameParser->parse(trim($name));

    // Check if the contributor exists.
    $query = \Drupal::entityQuery('bibcite_contributor')
      ->accessCheck(FALSE)
      ->condition('first_name', $parsed['first_name'])
      ->condition('last_name', $parsed['last_name'])
      ->execute();
    $uid = reset($query);

    if (!empty($query)) {
      // Check if there's already an attached author with the same name.
      $user = \Drupal::entityTypeManager()->getStorage('user')->load($uid);
      $authors = $user->field_author?->referencedEntities();
      $author_labels = [];
      if (!empty($authors)) {
        foreach ($authors as $author) {
          $author_labels[$author->id()] = $author->label();
        }
      }
      if (in_array(trim($name), $author_labels)) {
        foreach ($author_labels as $key => $author_label) {
          if ($author_label == trim($name)) {
            return $key;
          }
        }
        return reset($author_id);
      }
      else {
        return reset($query);
      }
    }

    return is_int($uid) ? $uid : $this->createContributor($name);
  }
  /**
   * Add a publication to a user.
   */
  public function addPublicationToUser(User $user, int $pub_id) {
    $user_references = $user->field_references->getValue();
    $current_references = isset($user_references) ? array_column($user_references, 'target_id') : [];
    if (!in_array($pub_id, $current_references)) {
      $user->field_references[] = ['target_id' => $pub_id];
      $user->save();
    }
  }

  /**
   * Given a author name, create an author and retutn the id.
   *
   * @return int
   *   Return the id of the author.
   */
  public function createContributor($authorName) {
    $contributor = \Drupal::entityTypeManager()->getStorage('bibcite_contributor');
    $humanNameParser = new HumanNameParser();
    $parsed = $humanNameParser->parse($authorName);
    $parsed['type'] = 'entity_contributor';
    $c = $contributor->create($parsed);
    $c->save();
    $id = $c->id();
    return $id;
  }

}
